/* --------------------------------------------------------------------------------------------
 * Copyright (c) Remy Suen. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const dockerfile_ast_1 = require("dockerfile-ast");
const dockerDefinition_1 = require("./dockerDefinition");
const docker_1 = require("./docker");
class DockerHighlight {
    computeHighlightRanges(document, position) {
        let dockerfile = dockerfile_ast_1.DockerfileParser.parse(document.getText());
        let provider = new dockerDefinition_1.DockerDefinition();
        let location = provider.computeDefinition(document, position);
        let image = location === null ? dockerfile.getContainingImage(position) : dockerfile.getContainingImage(location.range.start);
        const highlights = [];
        if (location === null) {
            for (let instruction of dockerfile.getCOPYs()) {
                let flag = instruction.getFromFlag();
                if (flag) {
                    let range = flag.getValueRange();
                    if (range && range.start.line === position.line && range.start.character <= position.character && position.character <= range.end.character) {
                        let stage = flag.getValue();
                        for (let other of dockerfile.getCOPYs()) {
                            let otherFlag = other.getFromFlag();
                            if (otherFlag && otherFlag.getValue() === stage) {
                                highlights.push(vscode_languageserver_1.DocumentHighlight.create(otherFlag.getValueRange(), vscode_languageserver_1.DocumentHighlightKind.Read));
                            }
                        }
                        return highlights;
                    }
                }
            }
            for (const from of dockerfile.getFROMs()) {
                for (const variable of from.getVariables()) {
                    if (docker_1.Util.isInsideRange(position, variable.getNameRange())) {
                        const name = variable.getName();
                        for (const loopFrom of dockerfile.getFROMs()) {
                            for (const fromVariable of loopFrom.getVariables()) {
                                if (fromVariable.getName() === name) {
                                    highlights.push(vscode_languageserver_1.DocumentHighlight.create(fromVariable.getNameRange(), vscode_languageserver_1.DocumentHighlightKind.Read));
                                }
                            }
                        }
                        return highlights;
                    }
                }
            }
            for (let instruction of image.getInstructions()) {
                for (let variable of instruction.getVariables()) {
                    if (docker_1.Util.isInsideRange(position, variable.getNameRange())) {
                        let name = variable.getName();
                        for (let instruction of image.getInstructions()) {
                            if (!(instruction instanceof dockerfile_ast_1.From)) {
                                for (let variable of instruction.getVariables()) {
                                    if (variable.getName() === name) {
                                        highlights.push(vscode_languageserver_1.DocumentHighlight.create(variable.getNameRange(), vscode_languageserver_1.DocumentHighlightKind.Read));
                                    }
                                }
                            }
                        }
                        return highlights;
                    }
                }
            }
        }
        else {
            let definition = document.getText().substring(document.offsetAt(location.range.start), document.offsetAt(location.range.end));
            for (let from of dockerfile.getFROMs()) {
                let range = from.getBuildStageRange();
                if (range && range.start.line === location.range.start.line) {
                    highlights.push(vscode_languageserver_1.DocumentHighlight.create(location.range, vscode_languageserver_1.DocumentHighlightKind.Write));
                    for (let instruction of dockerfile.getCOPYs()) {
                        let flag = instruction.getFromFlag();
                        if (flag) {
                            if (flag.getValue() === definition) {
                                highlights.push(vscode_languageserver_1.DocumentHighlight.create(flag.getValueRange(), vscode_languageserver_1.DocumentHighlightKind.Read));
                            }
                        }
                    }
                    return highlights;
                }
            }
            for (let arg of image.getARGs()) {
                let property = arg.getProperty();
                // property may be null if it's an ARG with no arguments
                if (property && property.getName() === definition) {
                    highlights.push(vscode_languageserver_1.DocumentHighlight.create(property.getNameRange(), vscode_languageserver_1.DocumentHighlightKind.Write));
                }
            }
            for (let env of image.getENVs()) {
                for (let property of env.getProperties()) {
                    if (property.getName() === definition) {
                        highlights.push(vscode_languageserver_1.DocumentHighlight.create(property.getNameRange(), vscode_languageserver_1.DocumentHighlightKind.Write));
                    }
                }
            }
            for (let instruction of image.getInstructions()) {
                // only highlight variables in non-FROM instructions
                if (!(instruction instanceof dockerfile_ast_1.From)) {
                    for (const variable of instruction.getVariables()) {
                        if (variable.getName() === definition) {
                            highlights.push(vscode_languageserver_1.DocumentHighlight.create(variable.getNameRange(), vscode_languageserver_1.DocumentHighlightKind.Read));
                        }
                    }
                }
            }
            for (const arg of dockerfile.getInitialARGs()) {
                const property = arg.getProperty();
                if (property && docker_1.Util.rangeEquals(property.getNameRange(), location.range)) {
                    for (const from of dockerfile.getFROMs()) {
                        for (const variable of from.getVariables()) {
                            if (variable.getName() === definition) {
                                highlights.push(vscode_languageserver_1.DocumentHighlight.create(variable.getNameRange(), vscode_languageserver_1.DocumentHighlightKind.Read));
                            }
                        }
                    }
                }
            }
        }
        return highlights;
    }
}
exports.DockerHighlight = DockerHighlight;
